

/**
 * 添加一个对象
 * @object 对象
 * @return 无
 * **/
Array.prototype.dj_addObject = function (object) {
  this.push(object);
};

/**
 * 在指定位置插入一个对象
 * @object 对象
 * @index 下标
 * @return 无
 * **/
Array.prototype.dj_addObjectAtIndex = function (object,index){
  this.splice(index, 0, object);
};

/**
 * 添加一组对象
 * @array 数组
 * @return 无
 * **/
Array.prototype.dj_addObjectsFromArray = function (array){
  this.push.apply(this,array);
}

/**
 * 移除一个对象
 * @object 要移除的对象
 * @key 键  注释：object 是 object 类型有效
 * @value 值  注释：object 是 object 类型有效
 * @return 无
 * **/
Array.prototype.dj_removeObject = function (object){
  let length = this.length
  let index = 0;
  while(index < length){
    index = this.indexOf(object,index);
    if(index===-1){ break; }
    this.splice(index, 1);
    index++;
  }

}

/**
 * 清空数组
 * @return 无
 * **/
Array.prototype.dj_removeAllObjects = function (){
  this.splice(0,this.length);
}
/**
 * 移除第一个对象
 * @return 无
 * **/
Array.prototype.dj_removeFirstObject = function (){
  if (this.length > 0){ this.shift();}
}
/**
 * 移除最后一个对象
 * @return 无
 * **/
Array.prototype.dj_removeLastObject = function (){
  if (this.length > 0){ this.pop();}
}
/**
 * 根据下标移除对象
 * @return 无
 * **/
Array.prototype.dj_removeObjectAtIndex = function (index){
  if (this.length > index){this.splice(index,1);}
}

/**
 * 移除一组对象
 * @array 要移除的数组
 * @return 无
 * **/
Array.prototype.dj_removeObjectAtIndex = function (array){
  let mapper = this.filter(item => { return !array.includes(item); });
  this.dj_removeAllObjects();
  this.dj_addObjectsFromArray(mapper);
}

/**
 * 替换指定下标的对象
 * @object 要替换的对象
 * @index 指定下标
 * @return 无
 * **/
Array.prototype.dj_replaceObjectAtIndexWithObject = function (object,index){
  if (this.length > index){this.splice(index,1,object);}
}

/**
 * 获取数组的第一个对象
 * @return 有返回第一个对象  无返回 undefined
 * **/
Array.prototype.dj_firstObject = function (){
  if (this.length > 0){
    return this[0];
  }else {
    return undefined;
  }
}
/**
 * 获取数组的最后一个对象
 * @return 有返回最后一个对象  无返回 undefined
 * **/
Array.prototype.dj_lastObject = function (){
  if (this.length > 0){
    return this[this.length-1]
  }else {
    return undefined;
  }
}

/**
 * 获取某个对象的所有下标
 * @object 对象
 * @return 返回一个下标数组
 * **/
Array.prototype.dj_objectAtIndexes = function(object){
  let mapper = []
  let length = this.length
  let index = 0;
  while(index < length){
    index = this.indexOf(object,index);
    if(index===-1){ break; }
    mapper.push(index);
    index++;
  }
  return mapper;
}
/**
 * 获取某个对象的下标
 * @object 对象
 * @return 有就返回下标值 无返回-1
 * **/
Array.prototype.dj_objectAtIndex = function(object){
  let mapper = this.dj_objectAtIndexes(object)
  if (mapper.length > 0){
    return mapper[0];
  }else {
    return  -1;
  }
}

/**
 * 是否包含一个对象
 * @object 对象
 * @return 包含返回 true 不包含返回 false
 * **/
Array.prototype.dj_containObject = function(object){
  return JSON.stringify(this).indexOf(JSON.stringify(object))!=-1
}


/**
 * 拼接成一个字符串
 * @sepString 拼接字符
 * @return 包含一个字符串
 * **/
Array.prototype.dj_componentsJoinedByString = function(sepString = '') {
  return this.join(sepString);
}


/**
 * 比较
 * @property 属性名 数组中全是对象有效
 * @desc 排序方式
 * @return 无
 * **/
Array.prototype.dj_compare = function(property = '',desc = true) {
  if (property.length > 0){
    this.sort(dj_compare(property,desc));
  }else {
    this.sort();
  }
}

/**
 * 检索出Number数组中的最大值
 * @return Number
 * **/
Array.prototype.dj_checkOutMaxValue = function() {
  return  this.reduce((num1, num2) => {
    return num1 > num2 ? num1 : num2}
  );
}
/**
 * 检索出Number数组中的最小值
 * @return Number
 * **/
Array.prototype.dj_checkOutMinValue = function() {
  return  this.reduce((num1, num2) => {
    return num1 > num2 ? num2:num1 }
  );
}









/**
 * 深 copy
 * @return 一个新的数组 占用的是新内存
 * **/
Array.prototype.dj_copy = function () {
  return Object.dj_copy(this);
};







/**
 * 公共类方法
 * **/
export function dj_compare(key,desc) {
  return function(object, anotherObject) {
    let  firstValue = object[key];
    let  secondValue = anotherObject[key];
    let type = typeof object;
    if (type == 'number'){
      if (desc == true) {
        // 升序排列
        return firstValue - secondValue;
      } else {
        // 降序排列
        return secondValue - firstValue;
      }
    }else {
      let currentValue = firstValue.match(/^(.*?)(\d*)$/);
      let targetValue = secondValue.match(/^(.*?)(\d*)$/);
      if (currentValue[1] === targetValue[1]) {
        return parseInt(currentValue[2] || 0) - parseInt(targetValue[2] || 0);
      } else {
        return currentValue[1].localeCompare(targetValue[1]);
      }
    }
  }
}
